// Squawk messenger. 
// Copyright (C) 2019  Yury Gubich <blue@macaw.me>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef CORE_CACHE_H
#define CORE_CACHE_H

#include <map>
#include <set>

#include <QString>

#include <core/storage/storage.h>

namespace Core {

template <class K, class V>
class Cache
{
public:
    Cache(const QString& name);
    ~Cache();

    void open();
    void close();

    void addRecord(const K& key, const V& value);
    void changeRecord(const K& key, const V& value);
    void removeRecord(const K& key);
    V getRecord(const K& key) const;
    bool checkRecord(const K& key) const;

private:
    Core::Storage<K, V> storage;
    std::map<K, V>* cache;
    std::set<K>* abscent;
};

}

#include "cache.hpp"

#endif // CORE_CACHE_H
